{% if tree.report.success %}
    <li class="file">
      <a href="{{ local.setup['href_prefix'] }}{{ tree.report.target }}">
        {{ tree.report.target.as_posix()|basename }}
      </a>
      {% if tree.report.target != tree.from_source %}
          ({{ render_download(tree) }})
      {% endif %}
{% else %}
    <li class="file failed">
        {{ render_download(tree) }}
        <span class="error">[Error : {{ tree.report.error }}]</span>
{% endif %}
      <div class="readme">
        {{ render_readme(tree) }}
      </div>
    </li>
