<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">
<html>
  <head>
    <meta http-equiv="content-type" content="text/html; charset=utf-8">
    <title>Example of Évariste html renderer</title>
    <link rel="stylesheet" type="text/css" href="static/tree.css" media="screen">
  </head>
  <body>
    <div class="tree">
      {% include "tree.html" %}
    </div>
  </body>
</html>
